@@ -26,11 +26,11 @@ _LRESERVED_KEYS = list(set(itt.chain.from_iterable([
 
 _DDEF_PARAMS = {
     'ddata': {
-        'source': (str, ''),
-        'dim':    (str, ''),
-        'quant':  (str, ''),
-        'name':   (str, ''),
-        'units':  ((str, asunits.core.UnitBase), ''),
+        'source': {'cls': str, 'def': ''},
+        'dim':    {'cls': str, 'def': ''},
+        'quant':  {'cls': str, 'def': ''},
+        'name':   {'cls': str, 'def': ''},
+        'units':  {'cls': (str, asunits.core.UnitBase), 'def': ''},
     },
     'dobj': {
     },
@@ -427,7 +427,7 @@ def _remove_obj(
 
     if key is None:
 
-        # remove 
+        # remove
         del dobj0[which]
 
     # ------------
@@ -738,7 +738,7 @@ def _get_suitable_ref(
 
     # no match => create new ref
     else:
-        key = _generic_check._obj_key(d0=dref0, short=dshort['ref'])
+        lref = _generic_check._obj_key(d0=dref0, short=dshort['ref'])
 
     return lref, size
 
@@ -786,13 +786,19 @@ def _check_data_ref(
     # length match but unknown ref or size mismatch
     else:
         for ii, ss in enumerate(ddata[k0]['shape']):
+
+            ri = ddata[k0]['ref'][ii]
             if ddata[k0]['ref'][ii] not in dref0.keys():
                 if ddata[k0]['ref'][ii] not in dref_add.keys():
                     dref_add[ddata[k0]['ref'][ii]] = {'size': ss}
 
-            elif ss != dref0[ddata[k0]['ref'][ii]]['size']:
+            elif ss != dref0[ri]['size']:
                 msg = (
-                    f"Mismatching ref size and shape for ddata['{k0}']"
+                    f"Mismatching ref size and shape for ddata['{k0}']\n"
+                    f"\t- ddata['{k0}']['ref'] = {ddata[k0]['ref']}\n"
+                    f"\t- dref['{ri}']['size'] = {dref0[ri]['size']}\n"
+                    f"\t- ddata['{k0}']['data'].shape = {ddata[k0]['shape']}\n"
+                    f"\t- ind: {ii}"
                 )
                 raise Exception(msg)
 
@@ -1091,21 +1097,44 @@ def _harmonize_params(
         )
         lparams = set(lparams).intersection(lkeys)
 
+    # ----------------
+    # check ddefparams
+
+    if not isinstance(ddefparams, dict):
+        msg = f"Arg ddefparams must be a dict!\nProvided: {type(ddefparams)}"
+        raise Exception(msg)
+
+    lv0 = ['def', 'cls']
+    for k0, v0 in ddefparams.items():
+        c0 = (
+            isinstance(v0, dict)
+            and all([ss in v0.keys() for ss in lv0])
+        )
+        if not c0:
+            msg = (
+                f"ddefparams['{k0}'] must be dict with keys:\n"
+                "\t- 'def': default value\n"
+                "\t- 'cls': expected class\n"
+                f"Provided: {v0}"
+            )
+            raise Exception(msg)
+
     # ----------------------------------------
     # check param types and set default values
 
     dfail = {}
     for k0, v0 in ddefparams.items():
+
         for k1, v1 in dd.items():
 
             # Set to default if None
             if v1.get(k0) is None:
-                dd[k1][k0] = v0[1]
+                dd[k1][k0] = v0['def']
 
             # Check type if already included
-            elif not isinstance(dd[k1][k0], v0[0]):
+            elif not isinstance(dd[k1][k0], v0['cls']):
                 dfail[k0] = (
-                    f" expected {v0[0]} vs "
+                    f" expected {v0['def']} vs "
                     f"type({dd_name2}['{k1}']['{k0}']) = {type(dd[k1][k0])}"
                 )
 
@@ -1118,7 +1147,7 @@ def _harmonize_params(
         )
         raise Exception(msg)
 
-    # set 
+    # set
     for k0 in lparams:
         for k1, v1 in dd.items():
             dd[k1][k0] = dd[k1].get(k0)
@@ -1130,8 +1159,10 @@ def _harmonize_params(
         for k1, v1 in dd.items():
             if v1.get('units') is not None and v1['units'] != '':
                 if isinstance(v1['units'], str):
-                    if hasattr(asunits, v1['units']):
-                        dd[k1]['units'] = eval(f"asunits.{v1['units']}")
+                    try:
+                        dd[k1]['units'] = asunits.Unit(v1['units'])
+                    except Exception as err:
+                        pass
 
     # -------------------
     # Check against dobj0
@@ -1863,7 +1894,7 @@ def _get_lparam(which=None, dd=None, for_show=None):
     return lparam
 
 
-def _show_get_fields(which=None, dobj=None, lparam=None, dshow=None):
+def _show_get_fields(which=None, lparam=None, dshow=None):
 
     # show dict
     if which not in dshow.keys():
