@@ -22,6 +22,7 @@ class DataStock3(DataStock2):
         keyX=None,
         keyY=None,
         keyZ=None,
+        keyU=None,
         ind=None,
         vmin=None,
         vmax=None,
@@ -54,6 +55,7 @@ class DataStock3(DataStock2):
             keyX=keyX,
             keyY=keyY,
             keyZ=keyZ,
+            keyU=keyU,
             ind=ind,
             vmin=vmin,
             vmax=vmax,
