@@ -63,8 +63,11 @@ def _add_ref(st=None, nc=None, nx=None, lnt=None):
 def _add_data(st=None, nc=None, nx=None, lnt=None):
 
     x = np.linspace(1, 2, nx)
-    lt = [np.linspace(0, 10, nt) for nt in lnt]
+    y = np.exp((x - 0.5)**2)
+    y[-5] = np.nan
+    lt = [np.linspace(1, 10, nt) for nt in lnt]
     lprof = [(1 + np.cos(t)[:, None]) * x[None, :] for t in lt]
+    lprof[0][10, -5] = np.nan
 
     # add data dependening on these references
     st.add_data(
@@ -76,6 +79,15 @@ def _add_data(st=None, nc=None, nx=None, lnt=None):
         ref='nx',
     )
 
+    st.add_data(
+        key='y',
+        data=y,
+        dim='distance',
+        quant='radius',
+        units='m',
+        ref='nx',
+    )
+
     for ii, nt in enumerate(lnt):
         st.add_data(
             key=f't{ii}',
@@ -150,11 +162,12 @@ def _add_obj(st=None, nc=None):
 
 class Test01_Instanciate():
 
-    def setup(self):
-        self.st = DataStock()
-        self.nc = 5
-        self.nx = 80
-        self.lnt = [100, 90, 80, 120, 80]
+    @classmethod
+    def setup_class(cls):
+        cls.st = DataStock()
+        cls.nc = 5
+        cls.nx = 80
+        cls.lnt = [100, 90, 80, 120, 80]
 
     # ------------------------
     #   Populating
@@ -179,15 +192,16 @@ class Test01_Instanciate():
 
 class Test02_Manipulate():
 
-    def setup(self):
-        self.st = DataStock()
-        self.nc = 5
-        self.nx = 80
-        self.lnt = [100, 90, 80, 120, 80]
+    @classmethod
+    def setup_class(cls):
+        cls.st = DataStock()
+        cls.nc = 5
+        cls.nx = 80
+        cls.lnt = [100, 90, 80, 120, 80]
 
-        _add_ref(st=self.st, nc=self.nc, nx=self.nx, lnt=self.lnt)
-        _add_data(st=self.st, nc=self.nc, nx=self.nx, lnt=self.lnt)
-        _add_obj(st=self.st, nc=self.nc)
+        _add_ref(st=cls.st, nc=cls.nc, nx=cls.nx, lnt=cls.lnt)
+        _add_data(st=cls.st, nc=cls.nc, nx=cls.nx, lnt=cls.lnt)
+        _add_obj(st=cls.st, nc=cls.nc)
 
     # ------------------------
     #   Add / remove
@@ -272,38 +286,176 @@ class Test02_Manipulate():
             dim='time',
         )
 
-    def test08_interpolate(self):
-        out = self.st.interpolate(
-            keys='prof0',
-            ref_keys=None,
-            ref_quant=None,
-            pts_axis0=[1.5, 2.5],
-            pts_axis1=[1., 2.],
-            pts_axis2=None,
-            grid=False,
-            deg=2,
-            deriv=None,
-            log_log=False,
-            return_params=False,
+    def test08_domain_ref(self):
+
+        domain = {
+            'nx': [1.5, 2],
+            'x': (1.5, 2),
+            'y': [[0, 0.9], (0.1, 0.2)],
+            't0': {'domain': [2, 3]},
+            't1': {'domain': [[2, 3], (2.5, 3), [4, 6]]},
+            't2': {'ind': self.st.ddata['t2']['data'] > 5},
+        }
+
+        dout = self.st.get_domain_ref(domain=domain)
+
+        lk = list(domain.keys())
+        assert all([isinstance(dout[k0]['ind'], np.ndarray) for k0 in lk])
+
+    def test09_binning(self):
+
+        bins = np.linspace(1, 5, 10)
+        lk = [
+            ('y', None, 0),
+            ('y', 'nx', 0),
+            (None, 'nt0', 0),
+            ('prof0', 'nt0', 0),
+            ('prof0', 'x', 1),
+        ]
+
+        for (k0, kr, ax) in lk:
+            dout = self.st.binning(
+                keys=k0,
+                ref_key=kr,
+                bins=bins,
+            )
+
+            k0 = list(dout.keys())[0]
+            shape = list(self.st.ddata[k0]['data'].shape)
+            shape[ax] = bins.size - 1
+            assert dout[k0]['data'].shape == tuple(shape)
+
+    def test10_interpolate(self):
+
+        lk = ['y', 'y', 'prof0', 'prof0', 'prof0', '3d']
+        lref = [None, 'nx', 't0', ['nt0', 'nx'], ['t0', 'x'], ['t0', 'x']]
+        lax = [[0], [0], [0], [0, 1], [0, 1], [1, 2]]
+        lgrid = [False, False, False, False, True, False]
+        llog = [False, False, False, True, False, False]
+
+        x2d = np.array([[1.5, 2.5], [1, 2]])
+        x3d = np.random.random((5, 4, 3))
+        lx0 = [x2d, [1.5, 2.5], [1.5, 2.5], x2d, [1.5, 2.5], x3d]
+        lx1 = [None, None, None, x2d, [1.2, 2.3], x3d]
+        ldom = [None, None, {'nx': [1.5, 2]}, None, None, None]
+
+        zipall = zip(lk, lref, lax, llog, lgrid, lx0, lx1, ldom)
+        for ii, (kk, rr, aa, lg, gg, x0, x1, dom) in enumerate(zipall):
+
+            domain = self.st.get_domain_ref(domain=dom)
+
+            dout = self.st.interpolate(
+                keys=kk,
+                ref_key=rr,
+                x0=x0,
+                x1=x1,
+                grid=gg,
+                deg=2,
+                deriv=None,
+                log_log=lg,
+                return_params=False,
+                domain=dom,
+            )
+
+            assert isinstance(dout, dict)
+            assert isinstance(dout[kk]['data'], np.ndarray)
+            shape = list(self.st.ddata[kk]['data'].shape)
+            x0s = np.array(x0).shape if gg is False else (len(x0), len(x1))
+            if dom is None:
+                shape = tuple(np.r_[shape[:aa[0]], x0s, shape[aa[-1]+1:]])
+            else:
+                shape = tuple(np.r_[x0s, 39]) if ii == 2 else None
+            if dout[kk]['data'].shape != tuple(shape):
+                msg = str(dout[kk]['data'].shape, shape, kk, rr)
+                raise Exception(msg)
+
+    def test11_interpolate_common_refs(self):
+        lk = ['3d', '3d', '3d']
+        lref = ['t0', ['nt0', 'nx'], ['nx']]
+        lrefc = ['nc', 'nc', 'nt0']
+        lax = [[0], [0, 2], [2], [2]]
+        llog = [False, True, False]
+
+        # add data for common ref interpolation
+        nt0 = self.st.dref['nt0']['size']
+        nt1 = self.st.dref['nt1']['size']
+        nc = self.st.dref['nc']['size']
+        self.st.add_data(
+            key='data_com',
+            data=1. + np.random.random((nc, nt1, nt0))*2,
+            ref=('nc', 'nt1', 'nt0'),
         )
-        assert isinstance(out, dict)
-        assert isinstance(out['prof0'], np.ndarray)
+
+        lx1 = [None, 'data_com', None]
+        ls = [(5, 90, 100, 80), (5, 90, 100), (5, 100, 5, 90)]
+        lr = [
+            ('nc', 'nt1', 'nt0', 'nx'),
+            ('nc', 'nt1', 'nt0'),
+            ('nc', 'nt0', 'nc', 'nt1'),
+        ]
+
+        zipall = zip(lk, lref, lax, llog, lx1, lrefc, ls, lr)
+        for ii, (kk, rr, aa, lg, x1, refc, ss, ri) in enumerate(zipall):
+
+            dout, dparams = self.st.interpolate(
+                keys=kk,
+                ref_key=rr,
+                x0='data_com',
+                x1=x1,
+                grid=False,
+                deg=2,
+                deriv=None,
+                log_log=lg,
+                store=False,
+                return_params=True,
+                domain=None,
+                ref_com=refc,
+            )
+
+            assert isinstance(dout, dict)
+            assert isinstance(dout[kk]['data'], np.ndarray)
+
+            if not (dout[kk]['data'].shape == ss and dout[kk]['ref'] == ri):
+                lstr = [f'\t- {k0}: {v0}' for k0, v0 in dparams.items()]
+                msg = (
+                    "Wrong interpolation shape / ref:\n"
+                    f"\t- ii: {ii}\n"
+                    f"\t- keys: {kk}\n"
+                    f"\t- ref_key: {rr}\n"
+                    f"\t- x1: {x1}\n"
+                    f"\t- ref_com: {refc}\n"
+                    f"\t- log_log: {lg}\n"
+                    f"\t- key['ref']: {self.st.ddata[kk]['ref']}\n"
+                    f"\t- x0['ref']: {self.st.ddata['data_com']['ref']}\n"
+                    "\n"
+                    # + "\n".join(lstr)
+                    "\n"
+                    f"\t- Expected shape: {ss}\n"
+                    f"\t- Observed shape: {dout[kk]['data'].shape}\n"
+                    f"\t- Expected ref: {ri}\n"
+                    f"\t- Observed ref: {dout[kk]['ref']}\n"
+
+                )
+                raise Exception(msg)
+
+
+                # Not tested: float, store=True, inplace
 
     # ------------------------
     #   Plotting
     # ------------------------
 
-    def test09_plot_as_array(self):
+    def test12_plot_as_array(self):
         dax = self.st.plot_as_array(key='t0')
         dax = self.st.plot_as_array(key='prof0')
         dax = self.st.plot_as_array(key='3d')
         plt.close('all')
 
-    def test10_plot_BvsA_as_distribution(self):
+    def test13_plot_BvsA_as_distribution(self):
         dax = self.st.plot_BvsA_as_distribution(keyA='prof0', keyB='prof0-bis')
         plt.close('all')
 
-    def test11_plot_as_profile1d(self):
+    def test14_plot_as_profile1d(self):
         dax = self.st.plot_as_profile1d(
             key='prof0',
             key_time='t0',
@@ -312,7 +464,7 @@ class Test02_Manipulate():
         )
         plt.close('all')
 
-    def test12_plot_as_mobile_lines(self):
+    def test15_plot_as_mobile_lines(self):
 
         # 3d
         dax = self.st.plot_as_mobile_lines(
@@ -335,7 +487,7 @@ class Test02_Manipulate():
     #   File handling
     # ------------------------
 
-    def test13_copy_equal(self):
+    def test16_copy_equal(self):
         st2 = self.st.copy()
         assert st2 is not self.st
 
@@ -343,10 +495,10 @@ class Test02_Manipulate():
         if msg is not True:
             raise Exception(msg)
 
-    def test14_get_nbytes(self):
+    def test17_get_nbytes(self):
         nb, dnb = self.st.get_nbytes()
 
-    def test15_saveload(self, verb=False):
+    def test18_saveload(self, verb=False):
         pfe = self.st.save(path=_PATH_OUTPUT, verb=verb, return_pfe=True)
         st2 = load(pfe, verb=verb)
         # Just to check the loaded version works fine
